use std::rc::Rc;

#[derive(Debug, Clone)]
pub struct List<T> {
    head: Option<Rc<Node<T>>>,
}

#[derive(Debug, Clone)]
pub struct Node<T> {
    data: T,
    parent: Option<Rc<Node<T>>>,
}

impl<T> List<T> {
    pub fn new() -> Self {
        List { head: None }
    }

    pub fn push_front(&self, data: T) -> List<T> {
        List {
            head: Some(Rc::new(Node {
                data: data,
                parent: self.head.clone(),
            })),
        }
    }

    pub fn without_front(&self) -> List<T> {
        List {
            head: self.head.as_ref().map(|node| node.parent.clone()).flatten()
        }
    }

    pub fn get_data(&self) -> Option<&T> {
        self.head.as_ref().map(|rc_node| &rc_node.data)
    }

    pub fn is_empty(&self) -> bool {
        self.head.is_none()
    }
}
